"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DisabledTestRuleConfig_1 = require("../model/DisabledTestRuleConfig");
const TestBreakpointRuleConfig_1 = require("../model/TestBreakpointRuleConfig");
const tsfFoldersDisabledTestRule_1 = require("../tsfFoldersDisabledTestRule");
const tsfFoldersFileNamesRule_1 = require("../tsfFoldersFileNamesRule");
const tsfFoldersImportsBetweenPackagesRule_1 = require("../tsfFoldersImportsBetweenPackagesRule");
const tsfFoldersTestWithBreakpointRule_1 = require("../tsfFoldersTestWithBreakpointRule");
var ConfigFactory;
(function (ConfigFactory) {
    function createForBetweenPackages(options) {
        const config = create(options, tsfFoldersImportsBetweenPackagesRule_1.IMPORTS_BETWEEN_PACKAGES_RULE_ID);
        // TODO xxx validate a bit (at least importPath should be set, allowedToImport should refer to recognised importPath)
        return config;
    }
    ConfigFactory.createForBetweenPackages = createForBetweenPackages;
    function createForFilenames(options) {
        const config = createFromArguments(options, tsfFoldersFileNamesRule_1.FILE_NAMES_RULE_ID);
        config.casings = config["file-name-casing"];
        validate(config, "casings", tsfFoldersFileNamesRule_1.FILE_NAMES_RULE_ID);
        validate(config, "ignorePaths", tsfFoldersFileNamesRule_1.FILE_NAMES_RULE_ID);
        return config;
    }
    ConfigFactory.createForFilenames = createForFilenames;
    function createForDisabledTestRule(options) {
        // older config had just 'true':
        if (options.ruleArguments.length === 0) {
            return DisabledTestRuleConfig_1.getDefaultDisabledTestRuleConfig();
        }
        const config = createFromArguments(options, tsfFoldersDisabledTestRule_1.DISABLED_TEST_RULE_ID);
        validate(config, "ban", tsfFoldersDisabledTestRule_1.DISABLED_TEST_RULE_ID);
        validate(config, "includePaths", tsfFoldersDisabledTestRule_1.DISABLED_TEST_RULE_ID);
        return config;
    }
    ConfigFactory.createForDisabledTestRule = createForDisabledTestRule;
    function createForTestBreakpointRule(options) {
        // older config had just 'true':
        if (options.ruleArguments.length === 0) {
            return TestBreakpointRuleConfig_1.getDefaultBreakpointRuleConfig();
        }
        const config = createFromArguments(options, tsfFoldersTestWithBreakpointRule_1.TEST_BREAKPOINT_RULE_ID);
        validate(config, "debugTokens", tsfFoldersTestWithBreakpointRule_1.TEST_BREAKPOINT_RULE_ID);
        validate(config, "includePaths", tsfFoldersTestWithBreakpointRule_1.TEST_BREAKPOINT_RULE_ID);
        return config;
    }
    ConfigFactory.createForTestBreakpointRule = createForTestBreakpointRule;
    function validate(config, prop, ruleId) {
        if (config[prop] === undefined) {
            throw new Error(`invalid config for rule ${ruleId} - ${prop} is missing`);
        }
    }
    function create(options, ruleId) {
        if (options.length !== 1) {
            throw new Error(`tslint rule is misconfigured (${ruleId}) - options length is ${options.length}`);
        }
        const config = options[0].config;
        return config;
    }
    function createFromArguments(options, ruleId) {
        const args = options.ruleArguments;
        if (args.length !== 1) {
            throw new Error(`tslint rule is misconfigured (${ruleId}) - options.ruleArguments length is ${args.length}`);
        }
        const config = args[0];
        return config;
    }
})(ConfigFactory = exports.ConfigFactory || (exports.ConfigFactory = {}));
//# sourceMappingURL=ConfigFactory.js.map