"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const GraphCluster_1 = require("../../../graph/GraphCluster");
const GraphNode_1 = require("../../../graph/GraphNode");
class DotStyleGenerator {
    constructor(config, outputter) {
        this.config = config;
        this.outputter = outputter;
    }
    outputStyling() {
        this.outputGraphStyle();
        this.outputDefaultNodeStyling();
    }
    outputGraphNodeStyle(nodeType) {
        switch (nodeType) {
            case GraphNode_1.NodeType.Normal:
                break;
            case GraphNode_1.NodeType.External:
                this.outputStylingForExternalNode();
                break;
            case GraphNode_1.NodeType.Any:
                this.outputStylingForAnyNode();
                break;
            default:
                throw new Error(`unhandled NodeType ${nodeType}`);
        }
    }
    outputStylingForExternalNode() {
        this.outputter.outputLine("node [style=dashed]");
    }
    outputStylingForAnyNode() {
        this.outputStylingForExternalNode();
    }
    outputContainerStyle(clusterType) {
        this.outputPlaceLabelsAtTop();
        switch (clusterType) {
            case GraphCluster_1.ClusterType.AreaWithSubFolders:
                this.outputter.outputLine(`node [shape="${this.config.dot.subFolderShape}"]`);
                break;
            case GraphCluster_1.ClusterType.FromOptimization:
                this.outputOptimizedStyle();
                break;
            case GraphCluster_1.ClusterType.DiagramCluster:
                this.outputOptimizedStyle();
                break;
            case GraphCluster_1.ClusterType.Root:
            case GraphCluster_1.ClusterType.TopLevel:
                this.outputter.outputLine(`node [shape="${this.config.dot.packageShape}"]`);
                break;
            default:
                throw new Error(`unhandled ClusterType ${clusterType}`);
        }
    }
    outputOptimizedStyle() {
        this.outputter.outputLines([`color = gray`, `style=dashed`]);
    }
    outputPlaceLabelsAtTop() {
        this.outputter.outputLine("labelloc = t");
        this.outputter.outputLine("");
    }
    outputGraphStyle() {
        this.outputter.outputLine(`graph [`);
        this.outputter.increaseIndent();
        this.outputter.outputLines([`bgcolor="#FFFFFF"`, `fillcolor="#FFFFFF"`]);
        this.outputter.decreaseIndent();
        this.outputter.outputLine(`]`);
    }
    outputDefaultNodeStyling() {
        // ref: working_copy.dot
        this.outputter.outputLine("node [");
        this.outputter.increaseIndent();
        // ref: colorscheme = https://graphviz.gitlab.io/_pages/doc/info/colors.html
        this.outputter.outputLines([
            `labeljust="l"`,
            `colorscheme="${this.config.dot.colorScheme}"`,
            `style=filled`,
            `fillcolor=3`,
            `shape=record`
        ]);
        this.outputter.decreaseIndent();
        this.outputter.outputLines([
            `]`,
            ``,
            // setting both head, tail as normally only one will apply
            `edge [arrowhead="vee", arrowtail="vee", style=dashed, color="black"]`
        ]);
    }
}
exports.DotStyleGenerator = DotStyleGenerator;
//# sourceMappingURL=DotStyleGenerator.js.map