"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DocGeneratorBase_1 = require("../shared/DocGeneratorBase");
const SECTION_SEPARATOR = "_____";
class TextDocGenerator extends DocGeneratorBase_1.DocGeneratorBase {
    generateDoc(packageConfig) {
        this.outputter.outputLine(SECTION_SEPARATOR);
        packageConfig.checkImportsBetweenPackages.packages
            .filter(pkg => !pkg.isExternal && this.filter.isImportPathOkForFolder(pkg))
            .forEach(pkg => {
            const packageName = pkg.importPath;
            this.outputter.outputLine(`${packageName} - ${pkg.description}`);
            this.outputter.increaseIndent();
            this.outputAllowedImports(pkg);
            this.outputSubFolders(pkg.subFolders.filter(sub => this.filter.isImportPathOkForSubFolder(sub)));
            this.outputter.decreaseIndent();
        });
        this.outputter.outputLine(SECTION_SEPARATOR);
    }
    outputAllowedImports(pkg) {
        const allowedToImport = this.outputAllowedToImport(pkg.allowedToImport);
        this.outputter.outputLine(`--> ${allowedToImport}`);
    }
    outputAllowedToImport(allowed) {
        if (allowed.some(value => value === "*")) {
            return "(any)";
        }
        return allowed.length > 0 ? `${allowed.join(", ")}` : "(none)";
    }
    outputSubFolders(subFolders) {
        if (subFolders.length === 0) {
            this.outputter.outputLine("");
            return;
        }
        this.outputter.increaseIndent();
        this.outputter.outputLine("folders:");
        this.outputter.increaseIndent();
        subFolders.forEach(folder => {
            this.outputter.outputLine(`${folder.importPath} - ${folder.description}`);
            const allowedToImport = this.outputAllowedToImport(folder.allowedToImport);
            this.outputter.increaseIndent();
            this.outputter.outputLine(`--> ${allowedToImport}`);
            this.outputter.decreaseIndent();
            this.outputter.outputLine("");
        });
        this.outputter.decreaseIndent();
        this.outputter.decreaseIndent();
    }
}
exports.TextDocGenerator = TextDocGenerator;
//# sourceMappingURL=TextDocGenerator.js.map