"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Lint = require("tslint");
const ConfigFactory_1 = require("./config/ConfigFactory");
const GeneralRuleUtils_1 = require("./utils/GeneralRuleUtils");
exports.TEST_BREAKPOINT_RULE_ID = "tsf-folders-test-with-breakpoint";
class Rule extends Lint.Rules.AbstractRule {
    apply(sourceFile) {
        const config = ConfigFactory_1.ConfigFactory.createForTestBreakpointRule(this.getOptions());
        if (!GeneralRuleUtils_1.GeneralRuleUtils.isFileInPaths(sourceFile.fileName, config.includePaths)) {
            return [];
        }
        const walker = new StatementsWalker(sourceFile, this.getOptions(), config);
        this.applyWithWalker(walker);
        return walker.getFailures();
    }
}
exports.Rule = Rule;
class StatementsWalker extends Lint.RuleWalker {
    constructor(sourceFile, options, config) {
        super(sourceFile, options);
        this.config = config;
    }
    visitCallExpression(node) {
        const text = node.getText();
        if (this.config.debugTokens.some(token => text.startsWith(token))) {
            this.addFailureAtNode(node.getFirstToken(), GeneralRuleUtils_1.GeneralRuleUtils.buildFailureString("do not hard code breakpoints in the test", exports.TEST_BREAKPOINT_RULE_ID));
        }
        super.visitCallExpression(node);
    }
}
//# sourceMappingURL=tsfFoldersTestWithBreakpointRule.js.map